# encoding: utf-8

module RSeed
  class Server < Sinatra::Base

    # Sinatra Conf
    # ------------
    set :root, File.join(File.dirname(__FILE__), '..', '..')
    set :haml, :format => :html5
    set :server, 'thin'
    set :xsf_header, 'X-Accel-Redirect'

    before do
      cache_control :public, :must_revalidate, :max_age => 60
    end

    # ConstrucThor huhu
    # -----------------
    def initialize(*args, &blk)
      super
      RSeed::Config.instance_eval(&blk) if block_given?
      @config = Config
      @utils = Utils
      confFile = File.join(settings.root, 'config.yml')
      if File.exists? confFile
        YAML.load_file(confFile).each_value do |c|
          c.each {|key, value| @config.set(key,value)}
        end
      end
      @rtorrent = Rtorrent.new(@config.get("rpc_host"), @config.get("rpc_path"), @config.get("rpc_port"))
      File.open(@config.get('public_link_file_path'), 'a+'){} if not File.exist?(@config.get('public_link_file_path'))
    end

    # Helperz
    # -------
    helpers do
      include Rack::Utils
      include Sinatra::Xsendfile
      
      def host
        "http://#{request.host}#{request.port == 80 ? '' : ':' + request.port.to_s}"
      end

      def auth!
        if not auth?
          redirect '/signin'
        else
          true
        end
      end

      def auth?
        @user = request.cookies["user"]
        @session = request.cookies["session"]
        check = Session.valid?(@user, @session)
        if !check
          puts "I think we have a problem with this session : #{@user.inspect}, #{@session.inspect}"
        end
        check
      end

      def publicLink hash
        File.open(@config.get('public_link_file_path')) do |fd|
          fd.any? do |line|
            return true if line.include?(hash)
          end
        end
        return false 
      end
    end

    # Sessions
    # --------
    get '/signin' do
      if auth?
        redirect '/'
      else
        haml :signin
      end
    end

    post '/signin' do
      user, session = Session.signin(params[:username], params[:password])
      response.set_cookie("user", :value => params[:username], :path => '/', :expires => Time.now + 60*60*24*90)
      response.set_cookie("session", :value => session, :path => '/', :expires => Time.now + 60*60*24*90)
      redirect '/'
    end

    # Index
    # ----- 
    get '/' do
      if auth!
        tlist = @rtorrent.getAll
        haml :index, :locals => { :tlist => tlist }
      end
    end

    # DDL
    # ---
    get '/d/*/*' do |hash, filename|
      if publicLink(hash)
        @rtorrent.getFilesInfo(hash).each do |file| 
          if file.has_value?(URI.unescape(filename)) 
            content_type "application/octet-stream"
            x_send_file("/download/#{file["path"].sub(/#{Regexp.escape(@config.get("download_dir"))}/, '')}")
          end
        end
      end
    end

    get %r{/d/([a-zA-Z0-9]+)} do |hash|
      if publicLink(hash)
        haml :download, :locals => { 
          :torrentFileTree => @utils.renderTree(@utils.treePath(@rtorrent.getTorrentPath(hash)), hash),
          :tname => @rtorrent.getName(hash),
          :thash => hash
        }  
      else
        404
      end
    end

    # Public link generator
    # ---------------------
    post '/share' do
      if auth!
        fullhash = @rtorrent.getHash(params[:hash]).downcase
        if not publicLink(fullhash)
          File.open(@config.get('public_link_file_path'), 'a+') do |fd|
            fd.puts fullhash
          end
        end
        return { "url" => url("/d/#{fullhash}"), "name" => @rtorrent.getName(fullhash) }.to_json
      end
    end

    # Add Torrent File
    # ----------------
    get '/add' do
      haml :add
    end

    post '/add/:type' do
      case params[:type]
      when 'file'
        filename = "#{@config.get("data_path")}/#{params[:filename]}"
        encoding = Base64.decode64(params[:data]).to_s.encoding.name
        File.open("#{filename}", "w:#{encoding}") do |f|
          f.puts Base64.decode64(params[:data])
        end
        200 if @rtorrent.add("#{File.join(settings.root, filename)}")
      when 'link'
        return "Torrent added !" if @rtorrent.add(params[:link])
      when 't411'
        throw(:halt, [404, "Ya pas d'id wesh ...."]) if params['lien'] == nil
        if(URI(params['lien']).host == "www.t411.me")
          throw(:halt, [401, "Go conf user/password pour t411 !"]) if @config.get("t411_username") == nil or @config.get("t411_password") == nil
          id = /\d+/.match(Nokogiri::HTML(open(params['lien'])).css("a.btn.call").first.values[0]).to_s
          token = JSON.parse(RestClient.post "https://api.t411.me/auth", :username => @config.get("t411_username"), :password => @config.get("t411_password"))['token']
          torrentName = JSON.parse(RestClient.get "https://api.t411.me/torrents/details/#{id}", {'Authorization' => token})['rewritename']
          torrentFileContent = RestClient.get "https://api.t411.me/torrents/download/#{id}", {'Authorization' => token}
          torrentFile = @config.get("data_path") + "/" + torrentName + ".torrent"
          File.open(torrentFile, 'wb') {|f| f.write(torrentFileContent) }
          return @rtorrent.add(torrentFile)
        else
          torrentFile = @config.get("data_path") + "/" + File.basename(params['lien'])
          File.open(torrentFile, 'wb') {|f| f.write(open(params['lien']).read)}
        end
      else
        404
      end
    end
    
    # API
    # ---
    get '/api/:request' do
      content_type :json
      case params[:request]
      when 'get_updown_total'
        @rtorrent.getUpDown.to_json
      when 'get_list'
        @rtorrent.getAll.to_json
      when 'get_diskinfo'
        @rtorrent.diskInfo.to_json
      else
        502
      end
    end

    post '/api/:request' do
      case params[:request]
      when 'delete'
        if not params[:hash].nil?
          if @rtorrent.delete(@rtorrent.getHash(params[:hash]))
            return "OK"
          else
            return "Error"
          end
        else
          502
        end
      else
        502
      end
    end

  end
end
